import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

class disc
{
	final static float DEFAULT_RADIUS = 200.0f;
	final static float DEFAULT_BULGE_SPEED = 0.4f;
	final static int MODE_INACTIVE = -1;
	final static int MODE_ACTIVE = 0;
	final static int MODE_BULGING = 1;

	vertex pos, vel;        //position and velocity..
	float radius;           //radius
	float oRadius;          //outer radius
	float bulgeOffset;      //bulge for bulge animation
	boolean bulgeReverse;   //bulging in or out?
	float angle;            //current angle
	float angleVel;         //current rotation speed
	int mode;
	int iColor;             //disc color/type
	Color color;            //disc color
	Color lColor;           //shine spot color.. (shows rotation..)
	Color oColor;           //outer radius color..

	public boolean isActive(){ return mode == MODE_ACTIVE; }
	public boolean isInactive(){ return mode == MODE_INACTIVE; }
	public boolean isBulging(){ return mode == MODE_BULGING; }
	public boolean isSpinning(){ return angleVel != 0.0f; }

	public void startBulging(){ mode = MODE_BULGING; }

	//handle the bulge animation
	public void bulge(boolean continuous)
	{
		if(!bulgeReverse)
		{
			if((bulgeOffset += DEFAULT_BULGE_SPEED) >= 3.0f) bulgeReverse = true;
		}
		else
		{
			if((bulgeOffset -= DEFAULT_BULGE_SPEED) <= 0.0f)
			{
				bulgeOffset = 0.0f;
				bulgeReverse = false;

				//reset to normal mode unless we're continuously bulging
				if(!continuous) mode = MODE_ACTIVE;
			}
		}
	}

	//set the spin speed
	public void setSpinSpeed(float spd)
	{
		angleVel = spd;
	}

	//spin the disc
	public void spin()
	{
		angle += angleVel;
		if(angle >= (float)Math.PI*2) angle -= (float)Math.PI*2;
		if(angle < 0) angle += (float)Math.PI*2;
	}

	//stop spinning...
	public void stopSpinning()
	{
		angleVel = 0;
	}

	//handle all disc motion
	public void move()
	{
		pos.add(vel);
		//spin();
		//if(isBulging()) bulge(false);
	}
	public void move(boolean continuous)
	{
		pos.add(vel);
		if(!continuous) spin();
		if(isBulging()) bulge(continuous);
	}


	//set default values
	void setDefaults()
	{
		pos = new vertex();
		vel = new vertex();
		radius = DEFAULT_RADIUS;
		oRadius = 0.0f;
		angle = 0.0f;
		angleVel = 0.0f;
		color = Color.GRAY;
		lColor = Color.BLACK;
		oColor = Color.GRAY;
		mode = MODE_ACTIVE;
		iColor = -1;
		bulgeReverse = false;
		bulgeOffset = 0.0f;
	}
	
	//constructors
	public disc()
	{
		setDefaults();
	}
	public disc(vertex coords, vertex vCoords, Color col, Color lcol, Color ocol, float r, float or, float a, float av)
	{
		setDefaults();
		pos = new vertex(coords);
		vel = new vertex(vCoords);
		color = col;
		lColor = lcol;
		oColor = ocol;
		radius = r;
		oRadius = or;
		angle = a;
		angleVel = av;
	}
}
