class marble
{
	final static int MAX_COLORS = 10;
	final static int COLOR_NONE = -1;
	final static int COLOR_RANDOMIZED = 7;
	final static int COLOR_IMMOVABLE = 8;
	final static int COLOR_DEAD = 9;
	final static int COLOR_BOMB = 10;
	final static int NUM_BASE_COLORS = 6;
	final static int NUM_BOMB_ANIMS = 3;
	final static int ANIM_SPEED = 2;
	final static int MODE_INACTIVE = -1;
	final static int MODE_ACTIVE = 0;
	final static int MODE_SHIFTING_LEFT = 1;
	final static int MODE_SHIFTING_RIGHT = 2;
	final static int MODE_NEEDS_REMOVING = 3;
	final static int MODE_DEAD = 4;
	final static int MODE_EXPLODING = 5;
	final static int MODE_DEBUG = 10;
	final static int DEFAULT_RADIUS = 15;
	final static int DEFAULT_POINTS = 100; 
	final static int DEFAULT_EXPLODE_TIME = 20;    //how long the explosion lingers on-screen
	final static int MAX_MULTIPLIER = 5;

	vertex pos,vel;
	int mode;
	int color;
	int animCounter;
	int explodeTimer;
	boolean animRev;
	boolean returning;  //tells if we're going out or in...
	float radius;
	float spinAngle;
	float angle;
	float angleVel;
	float speed;
	int pointMultiplier;     //current point multiplier
	
	public boolean isInactive(){ return mode == MODE_INACTIVE; }
	public boolean isActive(){ return mode == MODE_ACTIVE; }
	public boolean isDead(){ return mode == MODE_DEAD; }
	public boolean isExploding(){ return mode == MODE_EXPLODING; }
	public boolean isWildcard(){ return color == COLOR_RANDOMIZED; }
	public boolean isBomb(){ return color == COLOR_BOMB; }
	public boolean needsRemoving(){ return mode == MODE_NEEDS_REMOVING; }
	public boolean isDebugTagged(){ return mode == MODE_DEBUG; }

	public void deactivate(){ mode = MODE_INACTIVE; }
	public void setToRemove(){ mode = MODE_NEEDS_REMOVING; }
	public void setDebugTag(){ mode = MODE_DEBUG; }
	public void setToImmovable(){ color = COLOR_IMMOVABLE; }
	public void setToWildcard(){ color = COLOR_RANDOMIZED; }
	public void setToExploding()
	{
		animCounter = 0;
		animRev = false;
		mode = MODE_EXPLODING;
		vel.zero();
	}
	public void kill()
	{
		mode = MODE_DEAD;
		color = COLOR_DEAD;
	}

	//returns this item's point value
	public int getPointValue()
	{
		if(isBomb()) return DEFAULT_POINTS*2;

		return (DEFAULT_POINTS/2)*(color+1)*pointMultiplier;
	}

	//increase the points multiplier
	public void increasePointMultiplier()
	{
		if(pointMultiplier < MAX_MULTIPLIER) pointMultiplier++;
	}

	//animate the food/bomb
	public void animate()
	{
		if(isBomb())
		{
			if(!isExploding())
			{
				//do forward animation
				if(!animRev)
				{
					animCounter++;
					if(animCounter >= ANIM_SPEED*NUM_BOMB_ANIMS-1)
					{
						animRev = true;
					}
				}
				//do reverse animation
				else
				{
					animCounter--;
					if(animCounter <= 0)
					{
						animRev = false;
						animCounter = 0;
					}
				}
			}
		}
		if(isExploding())	
		{
			//do forward animation
			if(!animRev)
			{
				animCounter++;
				if(animCounter >= ANIM_SPEED*NUM_BOMB_ANIMS-1)
				{
					animRev = true;
				}
			}
			//do reverse animation
			else
			{
				//let the explosion linger...
				explodeTimer--;
				if(explodeTimer <= 0)
				{
					animRev = false;
					animCounter = 0;
					explodeTimer = 0;

					//kill when the lingering explosion is over..
					deactivate();
				}
			}
		}
	}

	//get current animation frame
	public int getAnimFrame()
	{
		if(isBomb())
		{
			return animCounter/ANIM_SPEED;
		}

		//return frame 0 by default
		return 0;
	}

	//returns the color int value..
	public int getColor()
	{
		return (!isWildcard() ? color : (int)(Math.random()*NUM_BASE_COLORS));
	}

	//handle the spin stuff..
	public void spin()
	{
		angle = (float)vertex.angleMod(angle+angleVel);
	}
	
	//just bounces back (flips the velocity)
	public void bounce()
	{
		angle = (float)(angle+Math.PI);
		vel.x = speed*(float)Math.cos(angle);
		vel.y = speed*(float)Math.sin(angle);

		//do a quick move to move away from the bounceable wall/arc
		move();
		move();
		move();

		//toggle the returning flag
		returning = !returning;
	}

	//move the marble..
	public void move()
	{
		pos.add(vel);
		spin();
	}

	void setDefaults()
	{
		pos = new vertex();
		vel = new vertex();
		radius = DEFAULT_RADIUS;
		color = COLOR_NONE;
		angle = angleVel = 0.0f;
		speed = 0.0f;
		mode = MODE_ACTIVE;
		animCounter = 0;
		animRev = returning = false;
		explodeTimer = DEFAULT_EXPLODE_TIME;
		pointMultiplier = 1;
	}
	
	//constructors
	public marble()
	{
		setDefaults();
	}
	public marble(int col, float ang, float rad)
	{
		setDefaults();
		color = col;
		angle = ang;
		radius = rad;
	}
	public marble(vertex coords, vertex vCoords, int col, float ang, float rad)
	{
		setDefaults();
		pos = new vertex(coords);
		vel = new vertex(vCoords);
		speed = (float)vel.magnitude();
		color = col;
		angle = ang;
		radius = rad;
	}
	public marble(marble mb)
	{
		pos = new vertex(mb.pos);
		vel = new vertex(mb.vel);
		color = mb.color;
		angle = mb.angle;
		radius = mb.radius;
	}
}
