class player
{
	final static int MODE_INACTIVE = -1;
	final static int MODE_ACTIVE = 0;
	final static int MODE_SUCKING = 1;
	final static int MODE_CHOMPING = 2;
	final static int MODE_OUCH = 3;
	final static int MODE_DEAD = 4;
	final static int MAX_MARBLES = 256;
	final static int DEFAULT_MAX_PROJECTILES = 3;
	final static int MIN_MARBLE_MATCH_LENGTH = 4;
	final static float DEFAULT_PROJECTILE_SPEED = 5.0f;
	//final static float DEFAULT_MARBLE_ARC_RADIUS = (float)Math.PI/64;
	final static float DEFAULT_MARBLE_ARC_RADIUS = (float)Math.PI/32;
	final static float DEFAULT_ARC_RADIUS = DEFAULT_MARBLE_ARC_RADIUS*2;
	final static float DEFAULT_RADIUS = 24.0f;
	final static float DEFAULT_ANGLE_ACCEL = (float)Math.PI/256;
	final static float DEFAULT_ANGLE_DECCEL = DEFAULT_ANGLE_ACCEL*2;
	final static float MAX_ANGLE_SPEED = (float)Math.PI/32;
	final static int DEFAULT_OUCH_TIME = 20;
	final static int ANIM_SPEED = 2;
	final static int NUM_SUCK_ANIMS = 2;
	final static int MAX_SUCK_COUNT = NUM_SUCK_ANIMS*ANIM_SPEED;
	final static int NUM_CHOMP_ANIMS = 5;
	final static int MAX_CHOMP_COUNT = NUM_CHOMP_ANIMS*ANIM_SPEED;
	final static float RISE_SPEED = 0.5f;

	vertex pos, vel;          //position, velocity vectors
        String name = null;       //current name associated with this player
	marble[] pMarble = new marble[MAX_MARBLES];
	int[] clearList = new int[MAX_MARBLES];    //the marble slots currently in need of clearing
	int numToClear;           //# marbles currently in need of clearing..
        int mode;                 //current state
	int maxProjectiles;       //max projectiles at once..
	float projectileSpeed;    //firing speed..
	int numMarbles;           //# marbles in the ring..
	int maxMarbles;           //max marbles in the ring..
	float marbleRadius;       //marble size..
	float marbleArcRadius;    //radial spacing between marbles..
	float radius;             //radius..
	float arcRadius;          //arc-radius..
	float angle;              //current angle..
	float angleVel;           //rotation speed
	float angleAccel;         //rotational acceleration
	float destAngle;          //destination angle (for mouse control)
	int curDisc;              //current disc (might not use this..)
	int curStreak;            //current streak
	int bestStreak;           //current best streak
	long score;               //score..
	long marbleScore;         //total marbles cleared..
	int suckCounter;          //counter for sucking
	int chompCounter;         //counter for chomping
	int ouchCounter;          //counter for ouch
	boolean stopped;          //motion is stopped..
	boolean arcClearing;      //currently clearing arc...
	boolean isLastStand;      //is this our last stand?
	int riseState;            //for the rising states
	float riseOffset;         //rise Y offset

	public boolean isActive(){ return mode == MODE_ACTIVE; }
	public boolean isInactive(){ return mode == MODE_INACTIVE; }
	public boolean isSucking(){ return mode == MODE_SUCKING; }
	public boolean isChomping(){ return mode == MODE_CHOMPING; }
	public boolean isOuched(){ return mode == MODE_OUCH; }
	public boolean isDead(){ return mode == MODE_DEAD; }
	public boolean isRising(){ return riseState == 2; }
	public boolean isSettling(){ return riseState == 1; }
	public boolean marbleQueueFull(){ return numMarbles >= maxMarbles+1; }
	public boolean arcFilled(){ return arcRadius*2 >= Math.PI*2; }

	//increase current streak tally
	public void addToStreak()
	{
		if(++curStreak > bestStreak) bestStreak = curStreak;
	}

	//clear the current streak tally
	public void clearStreak()
	{
		curStreak = 0;
	}

	//clear arc
	public void clearArc()
	{
		arcRadius -= DEFAULT_ARC_RADIUS;
		if(arcRadius <= DEFAULT_ARC_RADIUS)
		{
			arcRadius = DEFAULT_ARC_RADIUS;
			arcClearing = false;
		}
	}

	//begin the arc-clearing sequence
	public void startArcClearing()
	{
		arcClearing = true;
	}

	//increase the arc radius
	public int expandArcRadius()
	{
		arcRadius += DEFAULT_MARBLE_ARC_RADIUS;

		//check to see if we've encompassed the whole circle
		if(arcRadius >= Math.PI*2.0f) return -1;

		//successful
		return 0;
	}
	//increase the arc radius by set amount
	public int expandArcRadius(float dr)
	{
		arcRadius += dr;

		//check to see if we've encompassed the whole circle
		if(arcRadius >= Math.PI*2.0f) return -1;

		//successful
		return 0;
	}

	//decrease the arc radius
	public int decreaseArcRadius()
	{
		arcRadius -= DEFAULT_MARBLE_ARC_RADIUS*2;

		//check to see if we've hit 0
		//if(arcRadius <= 0.0f) return -1;

		if(!isLastStand)
		{
			//else if(arcRadius <= 0.0f) return -2;
			if(arcRadius <= 0.0f)
			{
				System.out.println("Reached first 0.0. This is our last stand.");
				arcRadius = 0.0f;
				isLastStand = true;
				return 0;
			}
		}
		else
		{
			if(arcRadius <= 0.0f)
			{
				System.out.println("Reached second 0.0. You're not a living organism anymore.");
				arcRadius = 0.0f;
				isLastStand = false;
				return 1;
			}
		}

		//successful
		return 0;
	}

	//decrease the arc radius by set amount
	public int decreaseArcRadius(float dr)
	{
		float ratio = 0;

		if(arcRadius <= (float)Math.PI) ratio = 1.0f;   //if the arm span is less than half the circle, don't change the decrement
		else ratio = arcRadius/(float)Math.PI;          //otherwise make it decrease slightly more the longer the arc

		arcRadius -= dr*ratio; //DEFAULT_MARBLE_ARC_RADIUS*2;

		//check to see if we've hit 0
		//if(arcRadius <= 0.0f) return -1;

		if(!isLastStand)
		{
			//else if(arcRadius <= 0.0f) return -2;
			if(arcRadius <= 0.0f)
			{
				System.out.println("Reached first 0.0. This is our last stand.");
				arcRadius = 0.0f;
				isLastStand = true;
				return 0;
			}
		}
		else
		{
			if(arcRadius <= 0.0f)
			{
				System.out.println("Reached second 0.0. You're not a living organism anymore.");
				arcRadius = 0.0f;
				isLastStand = false;
				return 1;
			}
		}
		//successful
		return 0;
	}

	//handle the ouch animation
	public void doOuch()
	{
		ouchCounter++;
		if(ouchCounter >= DEFAULT_OUCH_TIME)
		{
			mode = MODE_ACTIVE;
			ouchCounter = 0;
		}
	}

	//start the ouch
	public void startOuch()
	{
		ouchCounter = 0;
		mode = MODE_OUCH;
	}

	//see you on the other side, Food Boy...
	public void kill()
	{
		mode = MODE_DEAD;
		arcRadius = 0;
	}

	//handle the rising animation
	public void doRise()
	{
		if(riseState == 2)
		{
			riseOffset -= RISE_SPEED;
			if(riseOffset <= -radius/2)
			{
				riseState = 1;
			}
		}
	}
	public void doSettle()
	{
		if(riseState == 1)
		{
			riseOffset += RISE_SPEED;
			if(riseOffset >= 0)
			{
				riseState = 0;
				riseOffset = 0;
			}
		}
	}

	//handle the chomp
	public void doChomp()
	{
		if(++chompCounter >= MAX_CHOMP_COUNT)
		{
			chompCounter = 0;
			mode = MODE_ACTIVE;
		}
	}
	public void startChomp()
	{
		chompCounter = 0;
		mode = MODE_CHOMPING;
	}
	public void stopChomp()
	{
		chompCounter = 0;
		mode = MODE_ACTIVE;
	}

	//increment the suck counter till we hit the limit
	public void doSuck()
	{
		if(++suckCounter >= MAX_SUCK_COUNT) suckCounter = MAX_SUCK_COUNT;
	}
	//start sucking
	public void startSuck()
	{
		suckCounter = 0;
		mode = MODE_SUCKING;
	}
	//stop sucking
	public void stopSuck()
	{
		suckCounter = 0;
		mode = MODE_ACTIVE;
	}

	//sets animation frame, depending on mode
	public void setAnimFrame(int f)
	{
		if(isChomping()) chompCounter = f*ANIM_SPEED;
	}

	//get the current suck frame #
	public int getSuckFrame(){ return suckCounter/ANIM_SPEED; }

	//get the current chomp frame #
	public int getChompFrame(){ return chompCounter/ANIM_SPEED; }

	//get current animation frame
	public int getCurAnimFrame()
	{
		if(isActive()) return 0;
		if(isChomping()) return getChompFrame();
		if(isSucking()) return getSuckFrame();

		return 0;
	}

	//add this slot to the clear list..
	public void addToClearList(int slot)
	{
		clearList[numToClear++] = slot;
	}

	//add the marbles between these two slots to the clear list..
	public void addToClearList(int start, int end)
	{
		for(int i = start; i < end; i++) clearList[numToClear++] = i;
	}

	//reset the clear list..
	public void resetClearList()
	{
		numToClear = 0;
	}

	//checks each slot in the clear list to see if it yields a matching row..
	public int checkClearListForMatches()
	{
		int num = 0;

		for(int i = 0; i < numToClear; i++){
			num += checkForMarbleMatchesExt(clearList[i]);
			}
		return num;
	}

	//checks every slot in the player's queue for possible matches..
	public int checkForMarbleMatchesAll()
	{
		int num = 0;                //count the total matches obtained during this turn
		while(true){
			int numThis = 0;    //count the # of matches made during one sweep of the queue
			for(int i = 0; i < numMarbles; i += MIN_MARBLE_MATCH_LENGTH){
				numThis += checkForMarbleMatchesExt(i);
				}
			num += numThis;     //add the matches found during this sweep to the total match sum
			if(numThis <= 0) break;   //if no matches were found during this sweep, we're done checking, since there can't be any more..
                                                  //if some were found, we'll have to go through and check again for any new matches that may have arisen
			}
		return num;                 //return the total matches found
	}

	//remove a single marble..
	public void removeMarble(int slot)
	{
		for(int i = slot; i < numMarbles-1; i++){
			pMarble[i] = new marble(pMarble[i+1].color, arcRadius+i*marbleArcRadius*2+marbleArcRadius, pMarble[i].radius);
			}
		numMarbles--;
		marbleScore++;
	}

	//remove the marbles from between these index points..
	public void removeMarbles(int start, int end)
	{
		int len = end-start;
		System.out.println("(removing " + len + " marbles..)");

		int begin = end < numMarbles ? end : end-1;
		int stop = numMarbles;

		if(begin < stop-1){
			for(int i = begin; i < stop; i++){
				pMarble[i-len] = new marble(pMarble[i].color, arcRadius+(i-len)*marbleArcRadius*2+marbleArcRadius, pMarble[i].radius);
				}
			}
		numMarbles -= len;
		marbleScore += len;
	}

	//remove the marbles currently found in the clear list
	public void removeClearListMarbles()
	{
		for(int i = 0; i < numToClear; i++){
			removeMarble(clearList[i]);
			}
	}

	//mark these marbles for removal..
	public void markMarblesForRemoval(int start, int end)
	{
		int len = end-start;
//		for(int i = start; i < end; i++){
//			pMarble[i].setToRemove();
//			}
		pMarble[end < numMarbles ? end : end-1].setDebugTag();
		pMarble[numMarbles-1].setDebugTag();
	}

	//use the wildcard on this spot's color..
	public int useWildcardMarble(int slot)
	{
		//no marbles in queue, so nothing to match with
		if(numMarbles == 0) return -1;

		int total = 0;
		//get the color of the slot that was hit..
		int col = pMarble[slot].color;
		//go through and find all marbles in the queue that match that color..
		for(int i = 0; i < numMarbles; i++){
			int num = 0;
			while(i <= numMarbles-1 && pMarble[i].color == col){
				removeMarble(i);
				addToClearList(i+total+num);
				num++;
				}
			total += num;
			}
		System.out.println("" + total + " cleared by wildcard..");
		//return the total marbles cleared by this wildcard..
		return total;
	}

	//check the queue for new marble matches within the vicinity of this point..
	public int checkForMarbleMatches(int slot)
	{
		//just checking all the marbles for now..
		//change it later to only check the final few
		if(numMarbles < MIN_MARBLE_MATCH_LENGTH) return -1;

		//set the start and end points of the scan
		//make sure we don't go off the end of the queue, though..
		int start = (slot-MIN_MARBLE_MATCH_LENGTH < 0 ? 0 : slot-MIN_MARBLE_MATCH_LENGTH);
		int end = (slot+MIN_MARBLE_MATCH_LENGTH > numMarbles ? numMarbles : slot+MIN_MARBLE_MATCH_LENGTH);
		int cStart = -1, cEnd = -1;
		for(int i = start; i < end-1; i++){
			//don't check for matches with these colors..
			if(pMarble[i].color == marble.COLOR_DEAD || pMarble[i].color == marble.COLOR_IMMOVABLE) continue;

			if(pMarble[i].color == pMarble[i+1].color){
				if(cStart == -1){
					cStart = i;
					cEnd = i+1;
					}
				else cEnd++;
				}
			else{
				if(cEnd-cStart+1 >= MIN_MARBLE_MATCH_LENGTH) break;
				else cStart = cEnd = -1;
				}
			}

		if(cEnd-cStart+1 >= MIN_MARBLE_MATCH_LENGTH){
			String tempStr = "removing " + (cEnd-cStart+1) + " marbles: ";
			for(int i = cStart; i < cEnd+1; i++) tempStr += i + ",";
			System.out.println(tempStr);
			removeMarbles(cStart, cEnd+1);
			//markMarblesForRemoval(cStart, cEnd+1);

			int num = cEnd-cStart+1;

			//check for further matches..
			num += checkForMarbleMatches(cStart);

			//return the # of matches total in this sweep
			return num;
			}

		return 0;
	}

	//check the queue for new marble matches within the vicinity of this point..
	//returns the start slot of the match (if any), along with the length of the match chain..
	public int checkForMarbleMatchesExt(int slot)
	{
		//just checking all the marbles for now..
		//change it later to only check the final few
		if(numMarbles < MIN_MARBLE_MATCH_LENGTH) return -1;

		//set the start and end points of the scan
		//make sure we don't go off the end of the queue, though..
		int start = (slot-MIN_MARBLE_MATCH_LENGTH < 0 ? 0 : slot-MIN_MARBLE_MATCH_LENGTH);
		int end = (slot+MIN_MARBLE_MATCH_LENGTH > numMarbles ? numMarbles : slot+MIN_MARBLE_MATCH_LENGTH);
		int cStart = -1, cEnd = -1;
		for(int i = start; i < end-1; i++){
			//don't check for matches with these colors..
			if(pMarble[i].color == marble.COLOR_DEAD || pMarble[i].color == marble.COLOR_IMMOVABLE) continue;

			if(pMarble[i].color == pMarble[i+1].color){
				if(cStart == -1){
					cStart = i;
					cEnd = i+1;
					}
				else cEnd++;
				}
			else{
				if(cEnd-cStart+1 >= MIN_MARBLE_MATCH_LENGTH) break;
				else cStart = cEnd = -1;
				}
			}

		if(cEnd-cStart+1 >= MIN_MARBLE_MATCH_LENGTH){
			String tempStr = "removing " + (cEnd-cStart+1) + " marbles: ";
			for(int i = cStart; i < cEnd+1; i++) tempStr += i + ",";
			System.out.println(tempStr);
			removeMarbles(cStart, cEnd+1);
			addToClearList(cStart, cEnd+1);
			//markMarblesForRemoval(cStart, cEnd+1);

			int num = cEnd-cStart+1;

			//check for further matches..
			num += checkForMarbleMatchesExt(cStart);

			//return the # of matches total in this sweep
			return num;
			}

		return 0;
	}

	//insert this marble into this spot and move the rest of the line to the right..
	public int insertMarble(marble mb, int slot)
	{
		//if the queue is full, exit out..
		if(marbleQueueFull()) return -1;

		//if the # marbles is 0, slot 0 will be null, so we have to allocate it..
		if(numMarbles == 0){
			pMarble[slot] = new marble(mb.color, arcRadius+slot*marbleArcRadius*2+marbleArcRadius, mb.radius);
			numMarbles++;
			return 2;
			}

		//increase the length of the chain..
		numMarbles++;

		//shift the rest of the line right..
		for(int i = numMarbles-1; i > slot; i--){
			pMarble[i] = new marble(pMarble[i-1].color, pMarble[i-1].angle+marbleArcRadius*2, pMarble[i-1].radius);
			//pMarble[i] = pMarble[i-1] ;
			//pMarble[i].angle += marbleArcRadius*2;
			}

		//copy the new marble into the slot..
		pMarble[slot] = new marble(mb.color, arcRadius+slot*marbleArcRadius*2+marbleArcRadius, mb.radius);

		//check to see if queue is now full..
		//if(numMarbles >= maxMarbles) return 1;

		return 0;
	}

	//shift the entire marble queue either left or right..
	public void shiftMarblesLeft()
	{
		if(numMarbles <= 1) return;   //no reason to shift only one marble..

		//copy element #0
		marble tempMarble = pMarble[0];
		for(int i = 0; i < numMarbles-1; i++){
			pMarble[i] = pMarble[i+1] ;
			pMarble[i].angle -= marbleArcRadius*2;
			}
		pMarble[numMarbles-1] = tempMarble;
		pMarble[numMarbles-1].angle += marbleArcRadius*2*(numMarbles-1);
	}
	public void shiftMarblesRight()
	{
		if(numMarbles <= 1) return;   //no reason to shift only one marble..

		//copy last element
		marble tempMarble = pMarble[numMarbles-1];
		for(int i = numMarbles-1; i > 0; i--){
			pMarble[i] = pMarble[i-1] ;
			pMarble[i].angle += marbleArcRadius*2;
			}
		pMarble[0] = tempMarble;
		pMarble[0].angle -= marbleArcRadius*2*(numMarbles-1);
	}

	//update the marble angle placements..
	public void updateMarbleAngles(int start)
	{
		for(int i = start; i < numMarbles; i++){
			pMarble[i].angle = i*marbleArcRadius*2+marbleArcRadius;
			}
	}

	//add this marble to the end of the queue/line
	public int acquireMarble(int col)
	{
		//if(numMarbles > 0 && numMarbles >= maxMarbles) return -1;
		if(marbleQueueFull()) return -1;
		pMarble[numMarbles] = new marble(col, arcRadius+numMarbles*marbleArcRadius*2+marbleArcRadius, marbleRadius);
		numMarbles++;

		//check to see if queue is now full..
		//if(numMarbles > 0 && numMarbles >= maxMarbles) return 1;
		return 0;
	}
	public int acquireMarble(marble mb)
	{
		//if(numMarbles > 0 && numMarbles >= maxMarbles) return -1;
		if(marbleQueueFull()) return -1;
		pMarble[numMarbles] = new marble(mb.color, arcRadius+numMarbles*marbleArcRadius*2+marbleArcRadius, marbleRadius);
		numMarbles++;

		//check to see if queue is now full..
		//if(numMarbles > 0 && numMarbles >= maxMarbles) return 1;
		return 0;
	}

	//kill all marbles in the queue
	public void killMarbles()
	{
		for(int i = 0; i < numMarbles; i++){
			if(pMarble[i] == null || pMarble[i].isInactive()) continue;
			pMarble[i].kill();
			}
	}

	//come to a stop..
	public void slowDown()
	{
		if(angleVel < 0){
			if(angleVel <= -DEFAULT_ANGLE_DECCEL) angleVel += DEFAULT_ANGLE_DECCEL;
			else angleVel = 0;
			}
		else if(angleVel > 0){
			if(angleVel >= DEFAULT_ANGLE_DECCEL) angleVel -= DEFAULT_ANGLE_DECCEL;
			else angleVel = 0;
			}
	}

	//handle all motion
	public void move()
	{
		if(stopped){
			slowDown();
			}

		angle += angleVel;
		angle = (float)vertex.angleMod(angle);
	}

	public void rotateLeft(boolean m)
	{
		if(m){
			angleVel -= angleAccel;
			if(angleVel < -MAX_ANGLE_SPEED) angleVel = -MAX_ANGLE_SPEED;
			}
		else rotateLeftAbsolute();
	}
	public void rotateRight(boolean m)
	{
		if(m){
			angleVel += angleAccel;
			if(angleVel > MAX_ANGLE_SPEED) angleVel = MAX_ANGLE_SPEED;
			}
		else rotateRightAbsolute();
	}
	public void rotateLeftAbsolute()
	{
		angle -= angleAccel;
		if(angle < 0) angle += (float)Math.PI*2;
	}
	public void rotateRightAbsolute()
	{
		angle += angleAccel;
		if(angle >= (float)Math.PI*2) angle -= (float)Math.PI*2;
	}

	//set default values..
	void setDefaults()
	{
		pos = new vertex();
		vel = new vertex();
		mode = MODE_ACTIVE;
		name = "Player 1";
		radius = DEFAULT_RADIUS;
		angle = destAngle = 0.0f;
		angleVel = 0.0f;
		angleAccel = DEFAULT_ANGLE_ACCEL;
		score = marbleScore = 0;
		curDisc = 0;
		stopped = true;
		maxProjectiles = DEFAULT_MAX_PROJECTILES;
		projectileSpeed = DEFAULT_PROJECTILE_SPEED;
		numMarbles = 0;
		maxMarbles = 62;
		for(int i = 0; i < MAX_MARBLES; i++) clearList[i] = 0;
		numToClear = 0;
		marbleArcRadius = DEFAULT_MARBLE_ARC_RADIUS;
		//change this to calculate actual radius...
		marbleRadius = 10;
		//change this to actual arcradius later..
		arcRadius = DEFAULT_ARC_RADIUS;
		arcClearing = false;
		curStreak = bestStreak = 0;
		riseState = 2;
		riseOffset = radius;
		isLastStand = false;
	}

        //constructors..
        public player()
        {
		setDefaults();
        }
}
