//vector/vertex class
public class vertex
{
	float x,y,z,w;        //4-dimensional, just in case we need it..
	
	//constructor methods
	public vertex(){ }
	public vertex(float ax, float ay){ x = ax; y = ay; }
	public vertex(float ax, float ay, float az){ x = ax; y = ay; z = az; }
	public vertex(float ax, float ay, float az, float aw){ x = ax; y = ay; z = az; w = aw; }
	public vertex(vertex a)
	{
		if(a == null) x = y = z = 0;
		x = a.x; y = a.y; z = a.z; 
	}
	
	//rotate vertex using matrix transformation
        public void rotate(double rot_angle)
        {
                double vx_x = (double)x;
                double vx_y = (double)y;

		// [  cos[a]  | -sin[a]  ]  [ x ] = [ w ]
                // [  sin[a]  |  cos[a]  ]  [ y ]   [ z ]
                x = (float)(vx_x*Math.cos(rot_angle) - vx_y*Math.sin(rot_angle));
                y = (float)(vx_x*Math.sin(rot_angle) + vx_y*Math.cos(rot_angle));
        }
        
	//rotate vertex using matrix transformation (polar coordinate method)
        public void rotatePolar(double rot_angle)
        {
	        //add the rotation angle to theta
	        //we shouldn't have to mess with r
	        y += (float)rot_angle;
                
	        if(y < 0) y += (float)Math.PI*2.0f;
	        if(y >= (float)Math.PI*2.0f) y -= (float)Math.PI*2.0f;
        }
        
        //returns this vertex offset by another
        public vertex offset(vertex b)
        {
                return new vertex(x+b.x, y+b.y, z+b.z);
        }

        //returns the angle between this vertex and the one supplied
        public double angleBetween(vertex a)
        {
	        return angleMod(Math.atan2(y-a.y, x-a.x));
        }
        
        //keeps the supplied angle between 0 and 2pi
        public static double angleMod(double angle)
	{
		if(angle < 0) return angle+Math.PI*2.0;
		else if(angle >= Math.PI*2.0) return angle-Math.PI*2.0;
		
		return angle;
	}
        
        //add another vertex to this one
        public void add(vertex b)
        {
	        x += b.x;
	        y += b.y;
	        z += b.z;
	        w += b.w;
        }
        
        //subtract another vertex to this one
        public void subtract(vertex b)
        {
	        x -= b.x;
	        y -= b.y;
	        z -= b.z;
	        w -= b.w;
        }
        
        //find the distance to this vertex
        public float distance(vertex b)
        {
	        return (float)Math.sqrt((b.x-x)*(b.x-x) + (b.y-y)*(b.y-y));
        }
        
        //dot product of 2 vectors
	public double dot(vertex b)
	{
	        return (x*b.x + y*b.y);
	}

	//finds the cross product of two vectors
	public vertex cross(vertex b)
	{
		return new vertex(y*b.z-b.y*z, z*b.x-b.z*x, x*b.y-b.x*y);
	}
	
	//returns the length/magnitude of this vertex
	public double magnitude()	
	{
		return Math.sqrt(x*x+y*y);
	}
	
	//normalize this vector
	public float normalize()
	{
        	double mag = magnitude();

        	if(mag == 0) return 0.0f;

        	x /= mag;
        	y /= mag;

        	return (float)mag;
	}
	
	//zero out this vertex..
	public void zero()
	{
		x = y = z = w = 0.0f;
	}
	
	//finds the midpoint between this vertex and the one specified
	public vertex lineMidpoint(vertex b)
	{
		return new vertex((x+b.x)/2.0f, (y+b.y)/2.0f);
	}
	
	//set this vertex with some different values..
	public void set(float ax, float ay){ x = ax; y = ay; }
	public void set(float ax, float ay, float az){ x = ax; y = ay; z = az; }
	public void set(float ax, float ay, float az, float aw){ x = ax; y = ay; z = az; w = aw; }
	
	//check collision between two circles..
        public boolean circleCollision(vertex b, float a_rad, float b_rad)
        {
                float deltax, deltay, r;

                deltax = b.x-x;
                deltax *= deltax;
                deltay = b.y-y;
                deltay *= deltay;
                r = a_rad + b_rad;
                r *= r;
                if(r > deltax + deltay){
                        return true;
                        }
                return false;
        }
        
        //check collision between two rectangles centered at a point..
        public boolean rectCollision(vertex b, float aw, float ah, float bw, float bh){
                //check top left
                if(x-aw >= b.x-bw && y-ah >= b.y-bh && x-aw <= b.x+bw && y-ah <= b.y+bh) return true;
                //check top right
                if(x+aw >= b.x-bw && y-ah >= b.y-bh && x+aw <= b.x+bw && y-ah <= b.y+bw) return true;
                //check bottom left
                if(x-aw >= b.x-bw && y+ah >= b.y-bh && x-aw <= b.x+bw && y+ah <= b.y+bh) return true;
                //check bottom right
                if(x+aw >= b.x-bw && y+ah >= b.y-bh && x+aw <= b.x+bw && y+ah <= y+bh) return true;
                return false;
                }
        
        //convert the polar coords passed to appropriate 2d x,y values
        public void fromPolar(vertex pol)
	{
		//polar coords are (r, theta):(x, y)
		float r = pol.x, theta = pol.y;
		x = r*(float)Math.cos(theta);
		y = r*(float)Math.sin(theta);
	}
	
	//constrain this vertex to within these bounds (wraps)
	public void wrapWithinBounds(vertex a, vertex b)
	{
		if(x < a.x) x = b.x;
		else if(x > b.x) x = a.x;
		
		if(y < a.y) y = b.y;
		else if(y > b.y) y = a.y;
	}
	
	//keep the position vector within these bounds
	public void keepVelocityWithinBounds(vertex pos, float radius, vertex topL, vertex botR)
	{
		if(x < 0){
			if(pos.x-radius < topL.x) x = -x;
			}
		else if(x > 0){
			if(pos.x+radius > botR.x) x = -x;
			}
		if(y < 0){
			if(pos.y-radius < topL.y) y = -y;
			}
		else if(y > 0){
			if(pos.y+radius > botR.y) y = -y;
			}
	}

}

